#
#  ======== products.mak ========
#

SYSLINK_INSTALL_DIR = $(CURDIR)

# List of supported devices (choose one): OMAP3530, TI816X, TI814X, OMAPL1XX
#
DEVICE = _your_device_

# Master core (GPP) OS type (choose one): Linux, Qnx, Bios, Android
#
GPPOS = Linux

# SysLink HLOS driver options variable (choose one): 0 or 1
#
SYSLINK_BUILD_DEBUG=1
SYSLINK_BUILD_OPTIMIZE=0
SYSLINK_TRACE_ENABLE=1

# Set SDK type when building for a TI SDK kit (choose one): EZSDK or NONE 
#
SDK = _your_sdk_

# Define root dir to install SysLink driver and samples for target file-system
#
EXEC_DIR = _your_filesys_

# Optional: choose one to override DEVICE-specific default: COFF, ELF
#
LOADER =

# Optional: recommended to install all dependent components in one folder.
#
DEPOT = _your_depot_folder_

# If GPPOS is Android, set this to the location of your rowboat-android
# installation.
#
ANDROID_DIR = _your_android_install

# Define the product variables for the device you will be using.
#
ifeq ("$(DEVICE)","OMAP3530")
LINUXKERNEL             = $(DEPOT)/_your_linux_kernel_install_
CGT_ARM_INSTALL_DIR     = $(DEPOT)/_your_arm_code_gen_install_
CGT_ARM_PREFIX          = $(CGT_ARM_INSTALL_DIR)/bin/arm-none-linux-gnueabi-
IPC_INSTALL_DIR         = $(DEPOT)/_your_ipc_install_
BIOS_INSTALL_DIR        = $(DEPOT)/_your_bios_install_
XDC_INSTALL_DIR         = $(DEPOT)/_your_xdctools_install_
CGT_C64P_INSTALL_DIR    = $(DEPOT)/_your_c64p_code_gen_install_

else ifeq ("$(DEVICE)","TI816X")
LINUXKERNEL             = $(DEPOT)/_your_linux_kernel_install_
CGT_ARM_INSTALL_DIR     = $(DEPOT)/_your_arm_code_gen_install_
CGT_ARM_PREFIX          = $(CGT_ARM_INSTALL_DIR)/bin/arm-none-linux-gnueabi-
IPC_INSTALL_DIR         = $(DEPOT)/_your_ipc_install_
BIOS_INSTALL_DIR        = $(DEPOT)/_your_bios_install_
XDC_INSTALL_DIR         = $(DEPOT)/_your_xdctools_install_
CGT_C674_ELF_INSTALL_DIR= $(DEPOT)/_your_c674elf_code_gen_install_

# SYS/BIOS timer frequency (ti.sysbios.timers.dmtimer.Timer.intFreq)
TI81XXDSP_DMTIMER_FREQ  = 32768

# If SDK=NONE then below tools path is required
ifeq ("$(SDK)","NONE")
CGT_M3_ELF_INSTALL_DIR  = $(DEPOT)/_your_m3elf_code_gen_install_
endif

else ifeq ("$(DEVICE)","TI814X")
# Set one of the following OS variables
LINUXKERNEL             = $(DEPOT)/_your_linux_kernel_install_
QNX_INSTALL_DIR         = $(DEPOT)/_your_qnx_install_

CGT_ARM_INSTALL_DIR     = $(DEPOT)/_your_arm_code_gen_install_
CGT_ARM_PREFIX          = $(CGT_ARM_INSTALL_DIR)/bin/arm-none-linux-gnueabi-
IPC_INSTALL_DIR         = $(DEPOT)/_your_ipc_install_
BIOS_INSTALL_DIR        = $(DEPOT)/_your_bios_install_
XDC_INSTALL_DIR         = $(DEPOT)/_your_xdctools_install_
CGT_C674_ELF_INSTALL_DIR= $(DEPOT)/_your_c674elf_code_gen_install_

# SYS/BIOS timer frequency (ti.sysbios.timers.dmtimer.Timer.intFreq)
ifeq ("$(GPPOS)","Qnx")
TI81XXDSP_DMTIMER_FREQ  = 32768
else
TI81XXDSP_DMTIMER_FREQ  = 20000000
endif

# Shouldn't need to change these.
BIONIC_TARGETOBJS       = $(ANDROID_DIR)/out/target/product/ti814xevm/obj
BIONICTOOL_DIR          = $(ANDROID_DIR)/prebuilt/linux-x86/toolchain/arm-eabi-4.4.3
BIONIC_MISC             = $(BIONICTOOL_DIR)/arm-eabi/lib/ldscripts
BIONIC                  = $(ANDROID_DIR)/bionic

# If SDK=NONE then below tools path is required
ifeq ("$(SDK)","NONE")
CGT_M3_ELF_INSTALL_DIR  = $(DEPOT)/_your_m3elf_code_gen_install_
endif

# If GPPOS=Bios then below tools path is required
ifeq ("$(GPPOS)","Bios")
CGT_A8_ELF_INSTALL_DIR  = $(DEPOT)/_your_a8elf_code_gen_install_
endif

else ifeq ("$(DEVICE)","OMAPL1XX")
LINUXKERNEL             = $(DEPOT)/_your_linux_kernel_install_
CGT_ARM_INSTALL_DIR     = $(DEPOT)/_your_arm_code_gen_install_
CGT_ARM_PREFIX          = $(CGT_ARM_INSTALL_DIR)/bin/arm-none-linux-gnueabi-
IPC_INSTALL_DIR         = $(DEPOT)/_your_ipc_install_
BIOS_INSTALL_DIR        = $(DEPOT)/_your_bios_install_
XDC_INSTALL_DIR         = $(DEPOT)/_your_xdctools_install_

# If LOADER=ELF then below elf tools path is required else set C674 path 
ifeq ("$(LOADER)","ELF")
CGT_C674_ELF_INSTALL_DIR= $(DEPOT)/_your_c674elf_code_gen_install_
else
CGT_C674_INSTALL_DIR= $(DEPOT)/_your_c674_code_gen_install_
endif

else ifeq ($(MAKECMDGOALS), clean)
else ifeq ($(MAKECMDGOALS), .show-products)
else ifeq ($(MAKECMDGOALS), help)
else
    $(error DEVICE is set to "$(DEVICE)", which is invalid. Set this in <SysLink Install>/products.mak. Refer to the SysLink Install Guide for more information)
endif

# Use this goal to print your product variables.
.show-products:
	@echo "DEPOT                    = $(DEPOT)"
	@echo "DEVICE                   = $(DEVICE)"
	@echo "GPPOS                    = $(GPPOS)"
	@echo "SDK                      = $(SDK)"
	@echo "TI81XXDSP_DMTIMER_FREQ   = $(TI81XXDSP_DMTIMER_FREQ)"
	@echo "SYSLINK_BUILD_DEBUG      = $(SYSLINK_BUILD_DEBUG)"
	@echo "SYSLINK_BUILD_OPTIMIZE   = $(SYSLINK_BUILD_OPTIMIZE)"
	@echo "SYSLINK_TRACE_ENABLE     = $(SYSLINK_TRACE_ENABLE)"
	@echo "LOADER                   = $(LOADER)"
	@echo "SYSLINK_INSTALL_DIR      = $(SYSLINK_INSTALL_DIR)"
	@echo "IPC_INSTALL_DIR          = $(IPC_INSTALL_DIR)"
	@echo "BIOS_INSTALL_DIR         = $(BIOS_INSTALL_DIR)"
	@echo "XDC_INSTALL_DIR          = $(XDC_INSTALL_DIR)"
	@echo "LINUXKERNEL              = $(LINUXKERNEL)"
	@echo "QNX_INSTALL_DIR          = $(QNX_INSTALL_DIR)"
	@echo "CGT_ARM_PREFIX           = $(CGT_ARM_PREFIX)"
	@echo "CGT_C64P_INSTALL_DIR     = $(CGT_C64P_INSTALL_DIR)"
	@echo "CGT_C674_INSTALL_DIR     = $(CGT_C674_INSTALL_DIR)"
	@echo "CGT_C674_ELF_INSTALL_DIR = $(CGT_C674_ELF_INSTALL_DIR)"
	@echo "CGT_M3_ELF_INSTALL_DIR   = $(CGT_M3_ELF_INSTALL_DIR)"
	@echo "CGT_A8_ELF_INSTALL_DIR   = $(CGT_A8_ELF_INSTALL_DIR)"
	@echo "EXEC_DIR                 = $(EXEC_DIR)"
